/* 
	allow this stylesheet to compile as a myghty template, if convenient
	<%flags>inherit=None</%flags> 
*/

body {
	font-family: verdana, helvetica, sans-serif;
}

.header {
	border: 1px solid;
	padding: 10px;
	margin: 5px;
	text-align: center;
}

.content-body {
	margin: 30px 20px 20px 25px;
}


.left-column {
	float:left;
	margin: 5px;	
	padding: 20px;
	width: 120px;
	height: 50%;
	font-size:10px;
	background-color: #EEEEEE;
}

.footer {
	clear: left;
	border: 1px solid;
	padding: 20px;
	margin: 5px;
	font-size:12px;
	background-color: #BFBFBF;
	text-align: center;
}



