body {
    font-family: verdana, arial, sans-serif;
    font-size: 12pt;
}

.main {
	margin:10px;
	padding:10px;
}

.welcomeuser {
	font-size:11pt;
	font-weight:normal;
	text-align:right;
}

.leftnav {
	border: 1px solid;
	margin:10px;
	padding:10px;
	float:left;
	width:80px;
	height:250px;
}

.header {
	border: 1px solid;
	margin:10px;
	padding:10px;
	font-size:16pt;
	font-weight:bold;
}

.footer {
	border: 1px solid;
	padding:10px;
	text-align:center;
	margin:10px;
	clear:left;
}

.red {
	color:red;
}
