body, td {
	font-family: verdana, sans-serif;
	font-size: 12px;
}

body {
	background-color: #FDFBFC;
	margin:20px 20px 20px 20px;
}


p {
	margin-top:10px;
	margin-bottom:10px;
}

a {font-weight:normal; text-decoration:underline;}
a:link {color:#0000FF;}
a:visited {color:#0000FF;}
a:active {color:#0000FF;}
a:hover {color:#700000;}

.large {
	font-size:24px;
	font-weight:bold;
}

.medium {
	font-size:14px;
	font-weight:bold;    
}

.slogan {
	font-size:16px;
	font-weight:normal;
	font-style:italic;
}

.small {
	font-size:12px;
}

.grey {
	color:#BFBFBF;
}

.header {
	margin:10px;
}

.toolbar {
	margin:10px;
	padding:10px;
	border:1px solid;
}

.loginstatus {
	float:right;
}

.rightcontrols {
	float:right;
	padding-right:50px;
	text-align:right;
}

.main {
	margin:10px;
}

.confirm {
	width:300px;
}