body, td {
	font-family: arial, sans-serif;
	font-size: 12px;
}

body {
	background-color: #FDFBFC;
	margin:10px 10px 10px 10px;
}


p {
	margin-top:10px;
	margin-bottom:10px;
}

a {font-weight:normal; text-decoration:underline;}
a:link {color:#0000FF;}
a:visited {color:#0000FF;}
a:active {color:#0000FF;}
a:hover {color:#700000;}

.header {
	border:1px solid;
	height:100px;
	width:500px;
	margin-bottom:15px;
}


.headertext1 {
	font-size:24px;
	font-weight:bold;
}

.headertext2 {
	font-size:16px;
	font-weight:bold;
}

.headertext3 {
	font-size:14px;
}

.sidebar {
	border:1px solid;
	/*float:left;*/
	background-color:#bfbfbf;
	width:135px;
	height:100%;
	padding:0px 0px 0px 5px;
}

.container {
}

.main {
	 /*margin-left:150px;*/
	margin-left:10px;
}

.logo {
	float:left;
}

.titletext {
	font-family:arial, sans-serif;
	font-style:italic;
	font-weight:bold;
	font-size:32px;
	text-align:center;
	padding:20px;
}

.toolbar {
	float:right;
	margin-right:10px;
}

.breadcrumb {
	margin:10px 10px 20px 10px;
}

.boldbreadcrumblink {
	font-weight:bold;
}

.breadcrumblink {
}

.categorylist {
}

.categorylink {
}

.categoryitemcell {
}

.itemborder {
	background-color:#BFBFBF;
	padding:10px;
	margin:10px;
}

.itemimage {
	border:none;
}

.categoryitemname {
	text-align:center;
}



.itemdescription {
}

.itemname {
	font-weight:bold;
}

.variantname {
	padding:2px;
}

.variantoptions {
	padding:2px;
}

.cartheader {
	font-weight:bold;
}

.checkoutbox {
	margin:5px 5px 25px 5px;
}

.borderbox {
	margin:2px;
	padding:5px;
	border:1px solid;
}

.smallbutton {
	font-size:10px;
}

.smalllink {
	font-size:11px;
}



